<?php

namespace Modules\Backend\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Routing\Controller;
use Carbon\Carbon;
use Illuminate\Filesystem\Filesystem as File;

use Auth;

use Modules\Common\Http\Models\Company as Portal;
use Modules\Common\Http\Models\TreatmentPreview as Preview;
use Modules\Common\Http\Models\TreatmentPreviewRecord as Record;
use Modules\Backend\Http\Requests\PreviewFormRequest as PreviewRequest;


class BackendController extends Controller
{
    public function __construct(){

        $this->middleware('auth');

    }

    /**
     * Display a listing of the resource.
     * @return Response
     */

    public function index(Request $request)
    {
        $perPageCases = config("settings.general.items_per_page");

        $records = Record::where('treatment_preview_id','<>','null')
                    ->orderBy('updated_at','desc');

        $filter = $request->get("q"); 
        $class = new \stdClass();
        

        switch($filter)            
        {
            case "animation":
                $records->where("status","Pending");
                $class->filterAnimation = "active";
            break;
            case "attachments":
                $records->where("status","Animation Created");
                $class->filterAttachments = "active";
            break;
            case "upload":
                $records->where("status","Attachments Added");
                $class->filterUplaod = "active";
            break;
            default:
                $class->filterAll = "active";
            
        }

        $records = $records->paginate($perPageCases);
 
        return view('backend::preview.list', compact(['records', 'filter', 'class'] ));
    }

    /**
     * Show the form for creating a new resource.
     * @return Response
     */
    public function create()
    {
        try
        {
            $portals = Portal::getList();

        }catch (\Exception $e) {

            $bug = $e->getMessage();
            return redirect()->back()->with('error', $bug);

        }

        return view('backend::preview.create',compact('portals'));
    }

    /**
     * Store a newly created resource in storage.
     * @param Request $request
     * @return Response
     */
    public function store(PreviewRequest $request)
    {
       //

       try{

           $pre_revision = Preview::getRevision($request->distributor_id, $request->erp_case_number);
           $revision=$pre_revision+1;

           $portal =  Portal::find($request->distributor_id);

           if($pre_revision== 0){

                $preview = new Preview();
                $previewData = $request->only($preview->getFillable());
                $preview->addNewPreview($previewData);

            }else{

                $preview = Preview::where([
                                "distributor_id" => $request->distributor_id,
                                "erp_case_number" => $request->erp_case_number,
                            ])->orderBy("treatment_preview_id", "DESC")->first();

                $previewData = $request->only($preview->getFillable());

                $preview->fill($previewData);

                $preview->save();


           }

                $previewRecord = new Record();

                $recordData =  $request->only($previewRecord->getFillable());

                $storage_path = $this->getStoragePath($portal->storage_location, $request->erp_case_number, $revision);

                $ipr_file_path = $storage_path."ipr/".$request->ipr_form->getClientOriginalName();

                $this->upload($request->initial_files, $storage_path, "initial");

                $this->upload($request->final_files, $storage_path, "final");

                $this->upload($request->ipr_form, $storage_path, "ipr");

                $recordData = array_merge($recordData, [ "treatment_preview_id"  => $preview->treatment_preview_id,
                          "revision"              => $revision,
                          "storage_path"          => $storage_path,
                          "ipr_file_path"         => $ipr_file_path
                        ]);


                $previewRecord->addNewRecord($recordData);

               // dd($previewRecord->toArray());



        }catch (\Exception $e) {

            $bug = $e->getMessage();
            return redirect()->back()->with('error', $bug)->withInput();

        }

        return redirect()->back()->withSuccess("New treatment preview submitted.");

    }

    /**
     * Show the specified resource.
     * @param int $id
     * @return Response
     */
    public function show($id)
    {
        return view('backend::show');
    }

    /**
     * Show the form for editing the specified resource.
     * @param int $id
     * @return Response
     */
    public function edit($id)
    {
        return view('backend::edit');
    }

    /**
     * Update the specified resource in storage.
     * @param Request $request
     * @param int $id
     * @return Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     * @param int $id
     * @return Response
     */
    public function destroy($id)
    {
        //
    }

    private function getStoragePath($portal_location, $case_no , $revision, $attr = [] ){

        $cases_dir = config("settings.preview.storage.case_prefix");

        //path format {portal_location}/{cases_dir}/{case_no}/r{revision}/

        return sprintf("%s/%s/%s/r%s/",
                        $portal_location,
                         $cases_dir,
                         $case_no,
                         $revision);

    }


    private function upload($files, $path, $dir){

        if(gettype($files) == 'object'){

            $files = [$files];

        }

        $storage_dir = sprintf("%s/%s/%s",base_path(),$path,$dir);

        foreach($files as $file){

            $name = $this->formateStlName($file->getClientOriginalName());
            $file->move($storage_dir, $name);

        }


    }

    private function formateStlName($name){

        $name = preg_replace('/(?<=[0-9]_)\S+(?=.stl)/', '', $name);
        $name = str_replace("_.",".",$name);
        $name = str_replace("Tooth_UpperJaw_1", "Tooth_UpperJaw", $name);
        $name = str_replace("Tooth_LowerJaw_1", "Tooth_LowerJaw", $name);

    return $name;

    }

}
