<?php

namespace Modules\Backend\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Routing\Controller;

use Modules\Common\Http\Models\Company as Portal;
use Modules\Common\Http\Models\TreatmentPreview as Preview;
use Modules\Common\Http\Models\TreatmentPreviewRecord as Record;
use Modules\Common\Http\Models\TreatmentPreviewModification as Modification;
use Modules\Common\Http\Models\TreatmentPreviewStage as Stage;
use Carbon\Carbon;

class BackendPreviewController extends Controller
{

    public function __construct(){

        $this->middleware('auth');

    }


    /**
     * Show the specified resource.
     * @param int $token
     * @return Response
     */
    public function show($token)
    {

        try{

            $records = Record::where("token",$token);

            if($records->exists()){

                $record = $records->first();
                $preview = $record->toArray();

                $modificationUrl = (string)$record->StagesSaveUrl;

                return view("backend::preview.stages", compact([ 'preview', 'modificationUrl' ]));

            }

        }catch (\Exception $e) {
            $bug = $e->getMessage();
            return redirect()->back()->with('error', $bug)->withInput();
        }

    }


    /**
     * Show the specified resource.
     * @param int $token
     * @return Response
     */
    public function showPreview($token)
    {

        try{

        $records = Record::with("treatmentPreview")->where("token",$token);

        if($records->exists()){

            $record = $records->first();

            $preview = $record->toArray();
            $caseInfo = $record->treatmentPreview->toArray();
            $stagesRecord = Stage::where([[ "record_id", "=", $record->id], ["revision", "=",   $record->revision]])->first();
            $stages = $stagesRecord->stage;
            $stageNo = 0;

            $modifiedData = Null;

            if($record->modifications->count()>0){
                $modification = $record->modifications->last()->toArray();
                $modifiedData = json_decode($modification["modification"]);
            }

            $deletedToothData = $record->unity_tunity_deleted_teeth_data;

            $treatmentSteps = $record->getTreatmentSteps();

            if(!empty($preview["aligners_upper"])){
                $aligners[] = "U".$preview["aligners_upper"];
            }
            if(!empty($preview["aligners_lower"])){
                $aligners[] = "L".$preview["aligners_lower"];
            }

            $steps = implode(" - ",$aligners);

            $modificationURL = $record->BackendModifyUrl ;


            return view("backend::preview.hybrid",
                        compact([
                                   'preview',
                                    'caseInfo' ,
                                    'modificationURL',
                                    'stages',
                                    'stageNo',
                                    'modifiedData',
                                    'treatmentSteps',
                                    'deletedToothData',
                                    'steps'
                                ])
                    );

        }

        }catch (\Exception $e) {
            $bug = $e->getMessage();
            return redirect()->back()->with('error', $bug)->withInput();
        }

    }


    /**
     * Save stages data.
     * @param int $token
     * @param Request $request
     * @return Response
     */

    public function saveStages($token, Request $request){

        $records = Record::where("token",$token);

        if($records->exists() && $request->has("data") ){
            $record = $records->first();
            $data = json_decode($request->get("data"));
            $record_id = $record->id;
            $revision = $record->revision;
            $aligners_upper = $request->get("aligners_upper");
            $aligners_lower = $request->get("aligners_lower");

            $deleted_tooth = $request->get("removeTeethData");

            $stageData = [];
            $stageNo = 0;
            $stages = Stage::where([["record_id","=", $record_id],["revision", "=", $revision]])->delete();

            $stageData =[
                "record_id"   =>    $record_id,
                "revision"    =>    $revision,
                "aligners_upper"    =>      $aligners_upper,
                "aligners_lower"    =>      $aligners_lower,
                "stage"             => $request->get("data")

            ];

            $stages = Stage::create($stageData);
            $record->status = "Animation Created";

            $record->unity_deleted_teeth_data = $deleted_tooth;



            $record->save();

            return response()->json(["success" => "Stages has been saved"]);

        }
            return response()->json(["error" => "Stages has not"]);

    }


    /**
     * Save modifcation of treatment setup
     * @param Request $request
     * @param int $id
     * @return Json
     */
    public function saveModification(Request $request, $token)
    {
        //
        try{

            $records = Record::where("token",$token);


            if($records->exists()){
             $record = $records->first();

             $initialData = Null;
             $modification = json_decode($request->get("data"));

             if($record->modifications->count()>0  && isset($modification->attachment) ){

                $initialDataArray = $record->modifications->last()->toArray();
                $attachmentArray = $this->mergeAttachmentData($initialDataArray, $modification);
                $modification->attachment = json_encode($attachmentArray);

            }


            $json = json_encode((array)$modification);

            $modifiedData = Modification::create([
                                    "treatment_preview_id"=> $record->treatment_preview_id,
                                    "treatment_preview_record_id" => $record->id,
                                    "modification" => $json,

                                    ]);

            $record->unity_attachments_data = $json;
            $record->status = 'Attachments Added';
            $record->update();

            return response()->json(["success" => "Modificaiotn has saved"]);

            }




        }catch (\Exception $e) {

            $bug = $e->getMessage();
            return redirect()->back()->with('error', $bug)->withInput();

        }

        return response()->json(["error" => "Invalid Request"]);

    }


     /**
     * Merge Attachment data with previous data
     * @param Array $initialAttachment
     * @param Array  $attachmentArray
     * @return Array
     */


    protected function mergeAttachmentData($initialDataArray, $modification){


        $initialData = json_decode($initialDataArray["modification"]);
        $initialAttachment = (array) json_decode($initialData->attachment);
        $attachmentArray = (array) json_decode($modification->attachment);


        for($i=0; $i<count($attachmentArray["Items"]); $i++ ){

            $item = $attachmentArray["Items"][$i];
            $tootId = $item->attachedToothID;
             $position = $item->attachmentSavedTransform->position;

             if($position->x==0 && $position->y==0 && $position->z==0){
                 foreach($initialAttachment["Items"] as $j => $iniItem ){

                     $iniTootId = $iniItem->attachedToothID;

                     if($tootId == $iniTootId){
                         $attachmentArray["Items"][$i] = $iniItem;
                         $initialAttachment["Items"] = array_except($initialAttachment["Items"],[$j]);
                         break;

                     }

                 }

             }
         }

         return $attachmentArray;

    }


    /**
     * Show the form for variable.txt the specified resource.
     * @param int $id
     * @return Response
     */
    protected function variables($token)
    {

            try{

                $records = Record::where("token",$token);


                if($records->exists()){
                    $record = $records->first();

                    $logo = url("assets/preview/images/HSO-logo-w.png") ;
                    $bg = url("assets/preview/images/bg.png");
                    $model = $record->modelUrl;
                    $ipr = null;

                    if($record->ipr_file_path!=null){
                     $ipr = $record->IPRForm;
                    }

                    $attachments = $record->AttachmentData;
                    $decalData = $record->DecalData;

                    $variables = 'Models='. $model.'
                    Logo='. $logo .'
                    BG='. $bg .'
                    Copyright= '.Carbon::now()->year .' Henry Schein Orthodontics, All Rights Reserved.
                    IPR='. $ipr . '
                    ViewBG=
                    IPRHolders=
                    AfterAttachments=' . $attachments .'
                    AfterDecals=' . $decalData;


                    return $variables;

             }


            }catch (\Exception $e) {

                $bug = $e->getMessage();
                return redirect()->back()->with('error', $bug)->withInput();

            }
    }


}
