<?php

namespace Modules\Backend\Http\Requests;

use Illuminate\Foundation\Http\FormRequest;
use Illuminate\Contracts\Validation\Validator;
use Illuminate\Validation\ValidationException;
use \Illuminate\Support\MessageBag;

class PreviewFormRequest extends FormRequest
{
    /**
     * Get the validation rules that apply to the request.
     *
     * @return array
     */
    public function rules()
    {
        return [
            //
            'erp_case_number' => 'required',
            'initial_files' => 'required',
            'final_files' => 'required'            

        ];
    }

    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }

    /**
     * Configure the validator instance.
     *
     * @param  \Illuminate\Validation\Validator  $validator
     * @return void
     */
    public function withValidator(Validator $validator)
    {
        //validate stl files before upload

        $validator->after(function ($validator) {

            $intialFiles = $this->initial_files;   
            $finalFiles = $this->final_files;

            if(!$this->validateStls($intialFiles)){

                $validator->errors()->add('initial_files', 'The initial files must be slt.');
            }

            if(!$this->validateStls($finalFiles)){

                $validator->errors()->add('initial_files', 'The final files must be stl.');
            }
        });
    }

 
    /**
     * Handle a failed validation attempt.
     *
     * @param  \Illuminate\Contracts\Validation\Validator  $validator
     * @return void
     *
     * @throws \Illuminate\Validation\ValidationException
     */
    protected function failedValidation(Validator $validator)
    {
        if($this->wantsJson())
        {
            $response = redirect()
                ->back()
                ->with('message', 'Ops! Some errors occurred')
                ->withErrors($validator);
        }else{
         
           redirect()
            ->back()
            ->with([
        
                'success' => false,
                'message' => 'Ops! Some errors occurred',
                'errors' => $validator->errors()
            ])
            ->withErrors($validator->messages())
            ->withInput($this->all());
            
                 $response = redirect(route('preview.create'))
            
            ->with([
        
                'success' => false,
                'message' => 'Ops! Some errors occurred',
                'errors' => $validator->errors()
            ]);
        }
        
        throw (new ValidationException($validator, $response))
            ->errorBag($this->errorBag)
            ->redirectTo($this->getRedirectUrl());
    }

    private function validateStls($files){

        if(!$files){

            return false;

        }

        //For single single file validation

        if(gettype($files) == "object"){

            $ext = strtolower($files->getClientOriginalExtension());
            if($ext != "stl"){
              return false;
            }

        }    

        //For multiple files validations

        foreach($files as $file){

            $ext = strtolower($file->getClientOriginalExtension());

            if($ext != "stl"){
               
              return false;
            }

      }

      return true;


    }
}
