<?php

namespace Modules\Common\Http\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Crypt;


class TreatmentPreviewRecord extends Model
{
    //
    protected $table = 'tbl_application_treatment_preview_records';
    protected $primaryKey = 'id';
    protected $fillable = [
                            'id',
                            'treatment_preview_id',
                            'revision',
                            'theme',
                            'status',
                            'aligners_upper',
                            'aligners_lower',
                            'token',
                            'storage_path',
                            'preview_url',
                            'ipr_file_path',
                            'flag_patient_viewer',
                            'unity_ipr_placeholders',
                            'unity_attachments_data',
                            'unity_decal_data',
                            'unity_deleted_teeth_data',
                            'approved_at',
                            'created_at',
                            'updated_at'
                        ];
    protected $visible = [
                            'id',
                            'treatment_preview_id',
                            'revision',
                            'theme',
                            'status',
                            'aligners_upper',
                            'aligners_lower',
                            'PatientPreviewUrl',
                            'ModelUrl',
                            'AttachmentData',
                            'BackendAttachmentUrl',                            
                            'DecalData',                            
                            'ipr_file_path',
                            'unity_ipr_placeholders',
                            'IPRForm',
                            'created_at',
                            'updated_at'
                        ];

    protected $appends = [
                            'ErpNumber',
                            'CompanyId',
                            'CompanySlug',
                            'PublicUrl',
                            'ModelUrl',
                            'StagesUrl',
                            'StagesSaveUrl',
                            'PatientViwerUrl',
                            'IprUrl',
                            'ModifyUrl',
                            'BackendModifyUrl',
                            'BackendAttachmentUrl',
                            'AttachmentData',
                            'DecalData',
                            'IPRForm',

                        ];

    public function modifications(){

      return $this->hasMany('\Modules\Common\Http\Models\TreatmentPreviewModification', 'treatment_preview_record_id', 'id');

    }


    public function treatmentPreview(){

      return $this->belongsTo('\Modules\Common\Http\Models\TreatmentPreview', 'treatment_preview_id', 'treatment_preview_id');

    }

    public function stages(){

      return $this->hasMany('\Modules\Common\Http\Models\TreatmentPreviewStages', 'id', 'id');

    }

    public function addNewRecord(array $data = []){


        if(is_array($data)){

            $token = $this->getToken();
            $data["token"] =  $token;

            $record = TreatmentPreviewRecord::create($data);
            $record["preview_url"] = $record->getPreviewUrl();

            $record->save();

            return $record;
        }

    }

    public function getStagesUrlAttribute(){

        $stages_prefix = config("settings.backend.stages_route");
        $token = $this->token;
        $slug = "view";
        $url = sprintf("%s/%s/%s", $stages_prefix, $token, $slug );
        return url($url);

    }

    public function getStagesSaveUrlAttribute(){

      $stages_prefix = config("settings.backend.stages_route");
      $token = $this->token;
      $slug = "modify";
      $url = sprintf("%s/%s/%s", $stages_prefix, $token, $slug );
      return url($url);

  }
   
    private function getPreviewUrl(){

        $portal_prefix = $this->treatmentPreview->Company->preview_prefix;
        $token = $this->token;
        $slug = "view";
        $revision = "r".$this->revision;

        return sprintf("%s/%s/%s/%s",$portal_prefix, $token, $slug, $revision);

    }



   //Models Url Structure
   //{base_url}/{StoragePath}/
   public function getModelUrlAttribute(){

        return url($this->storage_path);

   }

   public function getPublicUrlAttribute(){

      return url($this->preview_url);

   }

   public function getIPRFormAttribute(){

    return ($this->ipr_file_path)?url($this->ipr_file_path):"";

   }

   public function getModifyUrlAttribute(){

    return url(sprintf("%s/%s",$this->preview_url, "modify"));

   }

   public function getBackendModifyUrlAttribute(){


    $stages_prefix = config("settings.backend.preview_route");
    $token = $this->token;
    $slug = config("settings.backend.modify_slug");;
    $url = sprintf("%s/%s/%s", $stages_prefix, $token, $slug );
    return url($url);


   }

   public function getBackendAttachmentUrlAttribute(){

    $stages_prefix = config("settings.backend.preview_route");
    $token = $this->token;
    $slug = config("settings.backend.preview_slug");;
    $url = sprintf("%s/%s/%s", $stages_prefix, $token, $slug );

    return url($url);

   }


   function getDecalDataAttribute(){

    $record = $this->attributes["unity_attachments_data"];

    if(!empty($record)){

      $recordObj = json_decode($record);

       if(isset($recordObj->decalData)) {

        return $recordObj->decalData;

       }

    }

   }


   function getDecalRecordAttribute(){

    $record = $this->attributes["unity_attachments_data"];

    if(!empty($record)){

      $recordObj = json_decode($record);

       if(isset($recordObj->decalData)) {

        return $recordObj->decalData;

       }
    }
   }

   function getAttachmentDataAttribute(){

    $record = $this->attributes["unity_attachments_data"];

    if(!empty($record)){

      $recordObj = json_decode($record);

       if(isset($recordObj->attachment)) {

          return $recordObj->attachment;

       }

    }

    //fix for the string gererated with new build
    return str_replace(['"{', '}"'],['{', '}'],stripslashes($record));

   }

   public function getTreatmentSteps(){

    return ($this->aligners_upper>$this->aligners_lower)?$this->aligners_upper : $this->aligners_lower;

   }

   public function getStagesLinkAttribute(){

    return url("backend/stages/".$this->attributes['token']."/view/". $this->attributes["revision"]);

   }

   public function getModelLinkAttribute(){

      return url(str_replace("/var/www/3dsetup.app/public_html/", "", $this->model_location ));

   }

   public function getToken(){

    $token = (string)Str::orderedUuid();

    if($this->where("token",$token)->exists()){
      return  $this->getToken();
    }

    return $token;

   }


}
