<?php

namespace Modules\Preview\Http\Controllers;


use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Routing\Controller;


use Modules\Common\Http\Models\Company as Portal;
use Modules\Common\Http\Models\TreatmentPreview as Preview;
use Modules\Common\Http\Models\TreatmentPreviewRecord as Record;
use Modules\Common\Http\Models\TreatmentPreviewModification as Modification;
use Modules\Common\Http\Models\TreatmentPreviewStage as Stage;
use Carbon\Carbon;

class PreviewController extends Controller
{


    /**
     * Show the specified resource.
     * @param int $id
     * @return Response
     */
    public function show(Request $request, $token )
    {
        try{

            $records = Record::with("treatmentPreview")->where("token",$token);

            if($records->exists()){

                $record = $records->first();

                $preview = $record->toArray();                
                $portal = $record->treatmentPreview->Company;
                $caseInfo = $record->treatmentPreview->toArray();
                $stagesRecord = Stage::where([[ "record_id", "=", $record->id], ["revision", "=",   $record->revision]])->first();
                $stages = $stagesRecord->stage;
                $stageNo = 0;
               
                $modifiedData = Null;

                if($record->modifications->count()>0){
                    $modification = $record->modifications->last()->toArray();
                    $modifiedData = json_decode($modification["modification"]);
                }

                $deletedToothData = $record->unity_tunity_deleted_teeth_data;

                $treatmentSteps = $record->getTreatmentSteps();

                if(!empty($preview["aligners_upper"])){
                    $aligners[] = "U".$preview["aligners_upper"];
                }
                if(!empty($preview["aligners_lower"])){
                    $aligners[] = "L".$preview["aligners_lower"];
                }

                $steps = implode(" - ",$aligners);

                $modificationURL = $record->ModifyUrl ;
                
                $portal = $record->treatmentPreview->Company;               
                
                return view($portal->view, 
                            compact([
                                        'preview',
                                        'portal',
                                        'caseInfo' ,
                                        'modificationURL',
                                        'stages',
                                        'stageNo',
                                        'modifiedData',
                                        'treatmentSteps',
                                        'deletedToothData',
                                        'steps'
                                        
                                    ])
                        );

            }

            }catch (\Exception $e) {
                $bug = $e->getMessage();
                return abort("500", $bug);
            }

    }


     /**
     * Show the form for variable.txt the specified resource.
     * @param int $id
     * @return Response
     */
    protected function variables($token)
    {

            try{

                $records = Record::where("token",$token);


                if($records->exists()){
                    $record = $records->first();
                    
                    $portal = $record->treatmentPreview->Company;

                    $logo = asset($portal->treatment_preview_logo) ;
                    
                    $bg = url("assets/preview/images/bg.png");
                    $model = $record->modelUrl;
                    $ipr = null;

                    if($record->ipr_file_path!=null){
                     $ipr = url($record->ipr_file_path);
                    }
                    
                    $footer_text = $portal->treatment_preview_footer_text;

                    $attachments = $record->AttachmentData;
                    $decalData = $record->DecalData;

                    $variables = 'Models='. $model.'
                    Logo='. $logo .'
                    BG='. $bg .'
                    Copyright=© '.Carbon::now()->year .' '. $footer_text . '
                    IPR='. $ipr . '
                    ViewBG=
                    IPRHolders='.$record->unity_ipr_placeholders.'
                    AfterAttachments=' . $attachments .'
                    AfterDecals=' . $decalData;

                    return $variables;

             }


            }catch (\Exception $e) {

                $bug = $e->getMessage();
                return redirect()->back()->with('error', $bug)->withInput();

            }
    }


     /**
     * Save modifcation of treatment setup
     * @param Request $request
     * @param int $id
     * @return Json
     */
    public function saveModification(Request $request, $token)
    {
        //
        try{

            $records = Record::where("token",$token);

            if($records->exists()){

             $record = $records->first();
             $record_id = $record->id;
             $treatment_preview_id = $record->treatment_preview_id;
            // ["treatment_preview_id", "=", $treatment_preview_id]

            $recordToModify = Modification::where("treatment_preview_id", $treatment_preview_id)
                                            ->where("treatment_preview_record_id", $record_id)->latest("created_at")->first();




            $modifications = json_decode($request->get("data"));

            $json = json_encode((array)$modifications);


            $data = [
                "treatment_preview_id"=> $record->treatment_preview_id,
                "treatment_preview_record_id" => $record->id,
                "modification" => $json,

            ];

            if($recordToModify!= null){
                $recordToModify->update($data);
            }else{
                Modification::create($data);
            }

            $record->save();

            return response()->json(["success" => "Modificaiotn has saved"]);

            }




        }catch (\Exception $e) {

            $bug = $e->getMessage();
            return redirect()->back()->with('error', $bug)->withInput();

        }

        return response()->json(["error" => "Invalid Request"]);

    }

}
