// A (possibly faster) way to get the current timestamp as an integer.
_now = Date.now || function () {
    return new Date().getTime();
};

// Returns a function, that, when invoked, will only be triggered at most once
// during a given window of time. Normally, the throttled function will run
// as much as it can, without ever going more than once per `wait` duration;
// but if you'd like to disable the execution on the leading edge, pass
// `{leading: false}`. To disable execution on the trailing edge, ditto.
_throttle = function (func, wait, options) {
    var context, args, result;
    var timeout = null;
    var previous = 0;
    if (!options) options = {};
    var later = function () {
        previous = options.leading === false ? 0 : _now();
        timeout = null;
        result = func.apply(context, args);
        if (!timeout) context = args = null;
    };
    return function () {
        var now = _now();
        if (!previous && options.leading === false) previous = now;
        var remaining = wait - (now - previous);
        context = this;
        args = arguments;
        if (remaining <= 0 || remaining > wait) {
            if (timeout) {
                clearTimeout(timeout);
                timeout = null;
            }
            previous = now;
            result = func.apply(context, args);
            if (!timeout) context = args = null;
        } else if (!timeout && options.trailing !== false) {
            timeout = setTimeout(later, remaining);
        }
        return result;
    };
};

//changes background image of teeth on both sides of an active tooth-line
var _colorTeeth = function(thisObj, colour, grey){
    if(!grey && colour == 'c'){ //change colour of a single teeth to celadon
        var toothImg = jQuery(thisObj).parents('.tooth').children('img');
        var oldSrc = toothImg.attr('src');

        if(!oldSrc.match(/g.png/)){
            var newSrc = oldSrc.replace(/[a-z].png/, 'c.png');
            toothImg.attr('src', newSrc);
        }
    }else if(!grey && colour == 'w'){ //change colour back to white
        var toothImg = jQuery(thisObj).parents('.tooth').children('img');
        var oldSrc = toothImg.attr('src');

        if(!oldSrc.match(/g.png/)){
            var newSrc = oldSrc.replace(/[a-z].png/, 'w.png');
            toothImg.attr('src', newSrc);
        }
    }else if(typeof($(thisObj).attr('class'))!="undefined"){ //change colour of adjected teeth

        var lineId = $(thisObj).attr('class').match(/[0-9]-[0-9]-?[0-9]?/);
        var state;
        var oldColour = colour;

        if(lineId[0].length == 3){
            if(lineId[0][0] == 1){
                for(var i=1; i<3; i++){
                    state = jQuery('.tooth-'+i+'-1').find('.tooth-state');
                    if((state.hasClass('missing') || state.hasClass('engagers')) && colour=='w')
                        colour = 'c';

                    var toothImg = jQuery('.tooth-'+i+'-1').find('img');
                    var oldSrc = toothImg.attr('src');
                    var newSrc = oldSrc.replace(/[a-z].png/, colour+'.png')
                    toothImg.attr('src', newSrc);
                    colour = oldColour;
                }
            }else{
                for(var i=3; i<5; i++){
                    state = jQuery('.tooth-'+i+'-1').find('.tooth-state');
                    if((state.hasClass('missing') || state.hasClass('engagers')) && colour=='w')
                        colour = 'c';

                    var toothImg = jQuery('.tooth-'+i+'-1').find('img');
                    var oldSrc = toothImg.attr('src');
                    var newSrc = oldSrc.replace(/[a-z].png/, colour+'.png')
                    toothImg.attr('src', newSrc);
                    colour = oldColour;
                }
            }
        }else{
            var whichPart = lineId[0].match(/[0-9]/);
            var whichTeeth =  lineId[0].match(/-[0-9]-[0-9]/)[0].match(/[0-9]-[0-9]/);
            var firstTooth = 'tooth-'+whichPart[0]+'-'+whichTeeth[0][0];
            var secondTooth = 'tooth-'+whichPart[0]+'-'+whichTeeth[0][2];

            var toothImg = $('.'+firstTooth+'').find('img');
            state = $('.'+firstTooth+'').find('.tooth-state');
            if((state.hasClass('missing') || state.hasClass('engagers')) && colour=='w')
                colour = 'c';

            var oldSrc = toothImg.attr('src');
            var newSrc = oldSrc.replace(/[a-z].png/, colour+'.png')
            toothImg.attr('src', newSrc);

            colour = oldColour;

            var toothImg = $('.'+secondTooth+'').find('img');
            state = $('.'+secondTooth+'').find('.tooth-state');
            if((state.hasClass('missing') || state.hasClass('engagers')) && colour=='w')
                colour = 'c';

            var oldSrc = toothImg.attr('src');
            var newSrc = oldSrc.replace(/[a-z].png/, colour+'.png')
            toothImg.attr('src', newSrc);
        }

    }

};


var _IPRForMissingTooth = function(qad, tooth, length){
    //returns teeth numbers for IPR in case a Tooth is missing
    //validate missing tooth are not last molars
    //in case of last molars, no need to find adjucent tooth
    cqad = qad;


    if( length==2 && qad == 1 ){

        cqad = _nextQd(qad);
        previousTooth = _getValidToothForMidline(qad, 0);
        length=3;
        tooth--;

    }else if(length==2 && qad == 3){

        cqad = _nextQd(qad);
        tooth = 1;
        previousTooth = _getValidToothForMidline(qad, 0);
        length = 3;

    }else if(qad%2==0){

        previousTooth = _getValidNextTooth(qad, tooth);
        previousTooth++; //fix for universal notation

        if(previousTooth==0){
           qad = _nextQd(qad);
            previousTooth = _getValidToothForMidline(qad, 0);

        }

        tooth++;

    }else{

        previousTooth = _getValidPreviousTooth(qad, tooth);
    }

    toothInUNotation = _getUniversalNot(cqad, tooth, length);

    preToothInUNotation = _getUniversalNot(qad, previousTooth, length);

    if(toothInUNotation > preToothInUNotation){

        return {first: preToothInUNotation, second: toothInUNotation}
    }else{

        return {first: toothInUNotation, second: preToothInUNotation}
    }




};

var _getValidToothForMidline= function(qad, current_tooth){

    previous_tooth = current_tooth +1;

inputName = "tooth_states["+qad+"-"+previous_tooth+"]";

if($("input[name='"+inputName+"']").val()=="missing"){

    _getValidPreviousTooth(qad, previous_tooth);

}

return previous_tooth;

}


var _getValidPreviousTooth= function(qad, current_tooth){

        previous_tooth = current_tooth +1;


    if(previous_tooth==0 || previous_tooth > 8){
        return null;
    }

    inputName = "tooth_states["+qad+"-"+previous_tooth+"]";


    if($("input[name='"+inputName+"']").val()=="missing"){

        _getValidPreviousTooth(qad, previous_tooth);

    }

    return previous_tooth;

}

var _getValidNextTooth= function(qad, current_tooth){

    next_tooth = current_tooth -1;

    inputName = "tooth_states["+qad+"-"+current_tooth+"]";

    if($("input[name='"+inputName+"']").val()=="missing"){

        _getValidNextTooth(qad, next_tooth);

    }

    return next_tooth;

}

var _getParamForIPRField= function(ele_tooth){

    let tooth_block = ele_tooth.attr("name");
    let values = tooth_block.match(/\d/g)
    let qad = parseInt(values[0]);
    let tooth = parseInt(values[1]);

    if(qad%2==0 && tooth==1 ){
        tooth = qad;
        qad -=1;
        tooth_next = 0;

    }
    else if(qad%2==0){

        tooth_next  = tooth;
        tooth -=1;

    }else{
        tooth_next  = tooth+1;
    }

    if(tooth_next==0){

        tooth_spaces = "tooth_spaces[" + qad + "-" + tooth + "]";
        tooth_space = "tooth-space-"+ qad + "-" + tooth ;
        tooth_line = "tooth-line-"+ qad + "-" + tooth ;
    }else{

        tooth_spaces = "tooth_spaces[" + qad + "-" + tooth + "-" + tooth_next +"]";
        tooth_space = "tooth-space-"+ qad + "-" + tooth + "-" + tooth_next;
        tooth_line = "tooth-line-"+ qad + "-" + tooth + "-" + tooth_next;
    }

    return {tooth_spaces:tooth_spaces, tooth_space:tooth_space, tooth_line: tooth_line};
}

var _addIPRField = function(ele_tooth){
    params = _getParamForIPRField(ele_tooth);
    tooth_space = params.tooth_space;
    tmp     = '<div id="'+tooth_space+'" class="tooth-space '+tooth_space+'">';
    tmp    += '<table class="text-wrapper">';
    tmp    +=  '<tbody><tr>';
    tmp    +=    '<td class="text"></td>';
    tmp    +=  '</tr>';
    tmp    +=  '</tbody></table>';
    tmp    +=  '</div>';

    $(".iprForm .form-teeth").append(tmp);


}

var _removeIPRField = function(ele_tooth){

    params = _getParamForIPRField(ele_tooth);
    tooth_space = params.tooth_space;
    tooth_spaces = params.tooth_spaces;
    tooth_line  = params.tooth_line;

    console.log(tooth_spaces);
    $("textarea[name='"+tooth_spaces+"']").remove();
    //$("textarea[name='"+tooth_spaces+"']").removeAttr("data-pos");
    $("."+tooth_line).removeClass("active");
    //$("."+tooth_line).remove();
    //$("#"+tooth_space).removeClass("active");
    $("#"+tooth_space).remove();

};

var _nextQd = function(qd){
    //return next quadrant if a incisor is missing
    if(qd > 2)
     return (((qd) %2) + 3 ) ;
    else
     return  ((qd) %2) + 1  ;

};

var _getUniversalNot = function(qad, tooth, length){

    let multiple = parseInt(Math.pow(-1,qad) * 9 + (1 - qad));
    let multiple2 = parseInt( Math.pow(-1,qad) * (28 -  qad ));;

    let factor = (qad>2)? multiple2 : multiple;
    let cutoff = (qad%2 == 0) ? 1 : 0;

    let ret = Math.abs((factor + parseInt(tooth)))

   // ret += cutoff;

   if(length==2){
        ret=(qad==1)? ret+2: ret+4;
    }

    return ret;


};

var _attachedTooth = function(qd, tooth){
    //returns tooth number with universal notation


    previousTooth = tooth-1;
    nextTooth = tooth+1;
    previousToothQd = qd;
    nextToothQd = qd;

    if(previous>8){
       previous = 1;

    }

    if(next > 8){

    }


};

var _posINNextQd = function(qd ){


};

(function($){
	'use strict';

	$(document).on('click', '.tooth', function(e) {
       var $state = $(this).find('.tooth-state > input');

       /*
		if ($state.is('[value="none"]')) {
			$state.val('engagers');
		} else if ($state.is('[value="engagers"]')) {
			$state.val('missing');
		} else {
			$state.val('none');
		}
        */

       if ($state.is('[value="none"]')) {
            $state.val('missing');
            _removeIPRField($state );
        } else {
            $state.val('none');
            _addIPRField($state )
        }

		$state.trigger('toothStateUpdate');
	})

	$(document).on('toothStateUpdate', '.tooth .tooth-state > input', function() {

        $(this)
			.parent()
			.removeClass('none missing engagers')
			.addClass($(this).val());
        if ($(this).val()!=='none'){
            _colorTeeth($(this), 'c');
        }else{
            _colorTeeth($(this), 'w')
        }


	});

	$(document).ready(function() {

		$('.tooth .tooth-state > input').trigger('toothStateUpdate');
	});

	$(document).on('click', '.form-teeth .tooth-space', function(e) {
		if ($(this).is('.active')) return;
		var id = $(this).attr('id').match(/tooth-space-(.*)/)[1];
		$(this).addClass('active');
		$('.form-teeth #tooth-line-' + id).addClass('active');
		var $text = $(this).find('.text');
        var name = 'tooth_spaces[' + id + ']';
        var values = name.match(/\d/g)
        var qad = parseInt(values[0]);
        let tooth = parseInt(values[1]);
        let length = values.length;
        var strIPR = "";

        let params = _IPRForMissingTooth(qad, tooth, length);

         strIPR = params.first+","+params.second;

        $text.html('<textarea name="' + name + '" class="iprIn" rows="1" data-pos="'+strIPR+'"></textarea>');
        console.log('<textarea name="' + name + '" class="iprIn" rows="1" data-pos="'+strIPR+'"></textarea>');
		$text.children('textarea').trigger('autoResize').focus();
        _colorTeeth($(this), 'g', true);

    });


	$(window).on('load', function() {
		$('.form-teeth .tooth-space.active').removeClass('active').trigger('click');
	});

    $('body').on('autoResize', '.form-teeth .tooth-space textarea', function () {
        var $this = $(this);

        var height = parseInt($this.css('height'));
        $this.css('height', height).addClass('no-transition');

        var diff = $this[0].scrollHeight - height;

        if (diff <= 0) {
            var _height = height;

            while ($this[0].scrollHeight <= height) {
                height = parseInt(height * 0.5)
                $this.css('height', height);
            }

            diff = $this[0].scrollHeight - height;
            $this.css('height', _height).height();
            $this.removeClass('no-transition');
        }

        $this.removeClass('no-transition');
        $this.css('height', height + diff);
    });

    $('body').on('keyup keypress focusin', '.form-teeth .tooth-space textarea', _throttle(function () {
        $(this).trigger('autoResize');
    }, 200));

    $(window).on('ready',function() {
    	$('.form-teeth .tooth-space textarea').trigger('autoResize');
    });

    $(document).on('ready', function() {
    	$('button').show();
    });

    $(document).on('blur','.form-teeth textarea', function() {
        if(!$(this).val()){
            var tspace = $(this).parents('.tooth-space').next();

            $(this).parents('.tooth-space').removeClass('active');
            tspace.removeClass('active');
            $(this).parents('.text').empty();
            _colorTeeth(tspace, 'w', true);
        }
    });

    $(".iprForm").on("keyup", "textarea",function(){
        let iprPosition = "";
        $(".iprForm textarea").each(function(){
            let position = $(this).data("pos"); 
            let value = $(this).val();
            iprPosition += position + "," + value + "|"
        })
        $("input[name='unity_ipr_placeholders']").val(iprPosition.slice(0,-1))
    });
    
})(jQuery);

